package org.gcube.dataharvest.harvester.sobigdata;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.SortedSet;

import org.apache.commons.lang.Validate;
import org.gcube.datacatalogue.ckanutillibrary.server.DataCatalogueFactory;
import org.gcube.datacatalogue.ckanutillibrary.server.DataCatalogueImpl;
import org.gcube.dataharvest.AccountingDataHarvesterPlugin;
import org.gcube.dataharvest.datamodel.HarvestedDataKey;
import org.gcube.dataharvest.harvester.BasicHarvester;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.trentorise.opendata.jackan.model.CkanGroup;


/**
 * The Class SoBigDataHarvester.
 *
 * @author Luca Frosini (ISTI-CNR)
 * @author Francesco Mangiacrapa at ISTI-CNR (francesco.mangiacrapa@isti.cnr.it)
 * May 24, 2018
 */
public abstract class SoBigDataHarvester extends BasicHarvester {

	private static Logger logger = LoggerFactory.getLogger(SoBigDataHarvester.class);


	//Added by Francesco
	private static final String GROUP_LABEL = "group";

	//Added by Francesco
	protected HashMap<String, String> mapSystemTypeToDBEntry;

	//Added by Francesco
	protected HashMap<String, String> mapCatalogueGroupToVRE;

	//Added by Francesco
	protected HashMap<String, String> mapWsFolderNameToVRE;

	//Added by Francesco
	protected String catalogueContext;

	//Added by Francesco
	private DataCatalogueFactory catalogueFactory;


//	public static String SECONDARY_TYPE_FORMAT = "$resource/Profile/SecondaryType/text() eq '%1s'";
//	public static String NAME_FORMAT = "$resource/Profile/Name/text() eq '%1s'";
//
//	public static String SECONDARY_TYPE = "ExcludingVREs";
//	public static String NAME = "AccountingHarvesters";



	public static final String SO_BIG_DATA_CONTEXT = "/d4science.research-infrastructures.eu/SoBigData";

	protected SortedSet<String> contexts;

	/**
	 * Instantiates a new so big data harvester.
	 *
	 * @param start the start
	 * @param end the end
	 * @param catalogueContext the catalogue context
	 * @param vreScopes the contexts
	 * @throws ParseException the parse exception
	 */
	public SoBigDataHarvester(Date start, Date end, String catalogueContext, SortedSet<String> vreScopes) throws ParseException {
		super(start, end);
		this.catalogueContext = catalogueContext;
		this.catalogueFactory = DataCatalogueFactory.getFactory();
		this.contexts = vreScopes;
		initMappingMaps();


//		this.excludedContexts = getExcludedContexts();
//		// Adding trailing slash to SO_BIG_DATA_CONTEXT to avoid to get VO
//		this.contexts = getSoBigDataContexts(contexts, SO_BIG_DATA_CONTEXT + "/");
//		logger.trace("Valid contexts are {}", contexts);
	}

	/**
	 * Inits the mapping maps.
	 */
	protected void initMappingMaps(){
		Properties properties = AccountingDataHarvesterPlugin.getProperties().get();
	    Set<String> keys = properties.stringPropertyNames();

	    mapSystemTypeToDBEntry = new HashMap<String, String>();
	    for (String key : keys) {
	      //System.out.println(key + " : " + properties.getProperty(key));
	      try{
	    	  HarvestedDataKey valueEnum = HarvestedDataKey.valueOf(key);
	    	  mapSystemTypeToDBEntry.put(properties.getProperty(key), valueEnum.name());
	      }catch(Exception e){
	    	  //silent
	      }
	    }

	    logger.info("Built from properties the mapping 'SystemType' to 'DB entry' : "+mapSystemTypeToDBEntry);

		//GET CATALOGUE'S GROUPS
		List<String> groups = loadGroupsFromCKAN(catalogueContext);
		//NORMALIZING THE GROUP NAME TO MATCH WITH VRE NAME
		Map<String,String> mapNormalizedGroups = normalizeGroups(groups);
		logger.debug("Map of Normalized Groups is: "+mapNormalizedGroups);

		//CREATING MAPPING BETWEEN (CATALOGUE GROUP NAME TO VRE NAME)
		mapCatalogueGroupToVRE = new HashMap<String, String>();
		//CREATING MAPPING BETWEEN (WS FOLDER NAME TO VRE NAME)
		mapWsFolderNameToVRE = new HashMap<String, String>();
		Set<String> normalizedGroups = mapNormalizedGroups.keySet();
		for (String context : contexts) {
			//logger.trace("Context is: " + context);
			String loweredVREName = context.substring(context.lastIndexOf("/") + 1, context.length()).toLowerCase();
			try {
				//logger.trace("vreName lowered is: " + loweredVREName);
				if (normalizedGroups.contains(loweredVREName)) {
					logger.debug("Normalized Groups matching the lowered VRE name: "+loweredVREName);
					// Creating the map with couple (catalogue group name, scope)
					mapCatalogueGroupToVRE.put(mapNormalizedGroups.get(loweredVREName), context);
				}

				mapWsFolderNameToVRE.put(loweredVREName, context);
			}
			catch (Exception e) {
				// silent
			}
		}

		logger.info("Map of Catalogue Groups To VRE is: "+mapCatalogueGroupToVRE);
		logger.info("Map of (lowered) Ws Folder Name To VRE is: "+mapWsFolderNameToVRE);

	}

	/**
	 * Normalize groups.
	 *
	 * @author Francesco Mangiacrapa
	 * @param groups the groups
	 * @return the map with couples (normalized group name, group name)
	 */
	private Map<String,String> normalizeGroups(List<String> groups) {
		Map<String,String> listNGroups = new HashMap<String,String>(groups.size());
		for (String group : groups) {
			String normalizedGroup = group;
			if(normalizedGroup.endsWith(GROUP_LABEL)){
				normalizedGroup = normalizedGroup.substring(0, normalizedGroup.length()-GROUP_LABEL.length());
			}
			normalizedGroup = normalizedGroup.replaceAll("-","");
			listNGroups.put(normalizedGroup.toLowerCase(), group);
		}
		return listNGroups;
	}


	/**
	 * Load groups from ckan.
	 *
	 * @param scope the scope
	 * @return the list
	 */
	private List<String> loadGroupsFromCKAN(String scope){
		List<String> groups = new ArrayList<String>();
		String ckanURL = "";
		try {
			DataCatalogueImpl utils = catalogueFactory.getUtilsPerScope(scope);
			ckanURL = utils.getCatalogueUrl();
			List<CkanGroup> theGroups = utils.getGroups();
			Validate.notNull(theGroups, "The list of Groups is null");
			for (CkanGroup ckanGroup : theGroups) {
				groups.add(ckanGroup.getName());
			}
		}
		catch (Exception e) {
			logger.error("Error occurred on getting CKAN groups for scope: "+scope+" and CKAN URL: "+ckanURL,e);
		}

		return groups;
	}

	/**
	 * Gets the map catalogue group to vre.
	 *
	 * @return the map catalogue group to vre
	 */
	public HashMap<String, String> getMapCatalogueGroupToVRE() {

		return mapCatalogueGroupToVRE;
	}


	/**
	 * @return the mapSystemTypeToDBEntry
	 */
	public HashMap<String, String> getMapSystemTypeToDBEntry() {

		return mapSystemTypeToDBEntry;
	}


	/**
	 * @return the mapWsFolderNameToVRE
	 */
	public HashMap<String, String> getMapWsFolderNameToVRE() {

		return mapWsFolderNameToVRE;
	}



//	/**
//	 * Gets the filtered contexts.
//	 *
//	 * @return the filtered contexts
//	 */
//	public SortedSet<String> getFilteredContexts() {
//		return contexts;
//	}

//	/**
//	 * Gets the filtering generic resource.
//	 *
//	 * @return the filtering generic resource
//	 */
//	protected SimpleQuery getFilteringGenericResource() {
//		return ICFactory.queryFor(GenericResource.class)
//				.addCondition(String.format(SECONDARY_TYPE_FORMAT, SECONDARY_TYPE))
//				.addCondition(String.format(NAME_FORMAT, NAME));
//	}

//	/**
//	 * Gets the generic resource.
//	 *
//	 * @return the generic resource
//	 */
//	protected GenericResource getGenericResource() {
//		SimpleQuery simpleQuery = getFilteringGenericResource();
//		List<GenericResource> res = ICFactory.clientFor(GenericResource.class).submit(simpleQuery);
//		if(res.size()==0) {
//			// At time of writing it should be an error but it can change in the future
//			logger.info("No {} for filtering contexts.", GenericResource.class.getSimpleName());
//			return null;
//		}
//		return res.get(0);
//	}

//	/**
//	 * Gets the excluded contexts.
//	 *
//	 * @return the excluded contexts
//	 */
//	public SortedSet<String> getExcludedContexts() {
//		SortedSet<String> excludedContexts = new TreeSet<>();
//
//		GenericResource genericResource = getGenericResource();
//		if(genericResource==null) {
//			return excludedContexts;
//		}
//
//		Element body = genericResource.profile().body();
//
//		/*
//		 * The following code parse an XML formatted as this
//		 *
//		 * <ResourceCatalogueHarvester>
//		 * 	<vres>
//		 * 		<vre>/d4science.research-infrastructures.eu/SoBigData/ResourceCatalogue</vre>
//		 * 		<vre>/d4science.research-infrastructures.eu/SoBigData/TagMe</vre>
//		 * 	</vres>
//		 * </ResourceCatalogueHarvester>
//		 *
//		 */
//
//		NodeList nodeList = body.getElementsByTagName(this.getClass().getSimpleName());
//		if(nodeList.getLength()==0) {
//			// At time of writing it should be an error but it can change in the future
//			logger.info("The body of the {} does not contains any information to filter contexts.", GenericResource.class.getSimpleName());
//		}
//
//		Element classNameElement = null;
//		for(int c=0; c<nodeList.getLength(); c++) {
//			Node node = nodeList.item(c);
//			if(node.getNodeType() == Node.ELEMENT_NODE) {
//				classNameElement = (Element) node;
//			}
//		}
//
//		Element vresElement = null;
//		NodeList vresNodeList = classNameElement.getElementsByTagName("vres");
//		for(int c=0; c<vresNodeList.getLength(); c++) {
//			Node node = vresNodeList.item(c);
//			if(node.getNodeType() == Node.ELEMENT_NODE) {
//				vresElement = (Element) node;
//			}
//		}
//
//		NodeList vreNodeList = vresElement.getElementsByTagName("vre");
//		for(int c=0; c<vreNodeList.getLength(); c++) {
//			Node node = vreNodeList.item(c);
//			if(node.getNodeType() == Node.ELEMENT_NODE) {
//				Element vreElement = (Element) node;
//				NodeList nl = vreElement.getChildNodes();
//				for(int i=0; i<nl.getLength(); i++) {
//					Node n = nl.item(i);
//					if(n.getNodeType() == Node.TEXT_NODE) {
//						excludedContexts.add(n.getNodeValue());
//					}
//				}
//
//			}
//		}
//
//		return excludedContexts;
//
//	}
//
//	/**
//	 * Filter context.
//	 *
//	 * @param context the context
//	 * @return true, if successful
//	 */
//	protected boolean filterContext(String context) {
//		if(excludedContexts.contains(context)) {
//			return true;
//		}
//		return false;
//	}

//	/**
//	 * Gets the so big data contexts.
//	 *
//	 * @param contexts the contexts
//	 * @param base the base
//	 * @return the so big data contexts
//	 */
//	protected SortedSet<String> getSoBigDataContexts(Set<String> contexts, String base) {
//		SortedSet<String> filteredContext = new TreeSet<>();
//		for(String context : contexts) {
//			if(context.startsWith(SO_BIG_DATA_CONTEXT)) {
//				if(!filterContext(context)) {
//					filteredContext.add(context);
//				}
//			}
//		}
//		return filteredContext;
//	}



}
